>From 1c1443e9251978cfd13ff30b63689aacb9e05cc7 Mon Sep 17 00:00:00 2001
From: Tadas Slotkus <devtadas@gmail.com>
Date: Sat, 20 Aug 2011 20:07:37 +0300
Subject: [carFl mem 44/65] rpci and rmmio functions: use static buffer

Registering functions now uses statically allocated buffer.
The size currently is 256 bytes.

Signed-off-by: Tadas Slotkus <devtadas@gmail.com>
---
 flash.h    |    2 ++
 flashrom.c |    1 +
 hwaccess.c |    9 +++++----
 pcidev.c   |    9 +++++----
 4 files changed, 13 insertions(+), 8 deletions(-)

diff --git a/flash.h b/flash.h
index 47b433b..41dbf0c 100644
--- a/flash.h
+++ b/flash.h
@@ -291,6 +291,8 @@ enum programmer {
 
 struct globals {
 	enum programmer programmer;
+	uint8_t undo_data_buf[256];
+	uint8_t *undo_data_free_ptr; //pointer to free space (for new undo data)
 };
 void init_globals(globals *g);
 
diff --git a/flashrom.c b/flashrom.c
index f425385..6c50566 100644
--- a/flashrom.c
+++ b/flashrom.c
@@ -51,6 +51,7 @@ void init_globals(globals *g)
 	#else
 	#error Please enable either CONFIG_DUMMY or CONFIG_INTERNAL
 	#endif
+	g->undo_data_free_ptr = g->undo_data_buf;
 }
 
 static char *programmer_param = NULL;
diff --git a/hwaccess.c b/hwaccess.c
index ee7c50c..ffcc0f9 100644
--- a/hwaccess.c
+++ b/hwaccess.c
@@ -218,17 +218,18 @@ int undo_mmio_write(void *p)
 		break;
 	}
 	/* p was allocated in register_undo_mmio_write. */
-	free(p);
+	//free(p);
 	return 0;
 }
 
 #define register_undo_mmio_write(g, a, c)					\
 {									\
 	struct undo_mmio_write_data *undo_mmio_write_data;		\
-	undo_mmio_write_data = malloc(sizeof(struct undo_mmio_write_data)); \
-	if (!undo_mmio_write_data) {					\
+	undo_mmio_write_data = (struct undo_mmio_write_data *)g->undo_data_free_ptr; \
+	g->undo_data_free_ptr += sizeof(struct undo_mmio_write_data); \
+	if (g->undo_data_free_ptr - g->undo_data_buf > 256) {\
 		msg_gerr("Out of memory!\n");				\
-		exit(1);						\
+		return;						\
 	}								\
 	undo_mmio_write_data->addr = a;					\
 	undo_mmio_write_data->type = mmio_write_type_##c;		\
diff --git a/pcidev.c b/pcidev.c
index 116b0ef..68da21c 100644
--- a/pcidev.c
+++ b/pcidev.c
@@ -287,17 +287,18 @@ int undo_pci_write(void *p)
 		break;
 	}
 	/* p was allocated in register_undo_pci_write. */
-	free(p);
+	//free(p);
 	return 0;
 }
 
 #define register_undo_pci_write(g, a, b, c) 				\
 {									\
 	struct undo_pci_write_data *undo_pci_write_data;		\
-	undo_pci_write_data = malloc(sizeof(struct undo_pci_write_data)); \
-	if (!undo_pci_write_data) {					\
+	undo_pci_write_data = (struct undo_pci_write_data *)g->undo_data_free_ptr; \
+	g->undo_data_free_ptr += sizeof(struct undo_pci_write_data); \
+	if (g->undo_data_free_ptr - g->undo_data_buf > 256) {\
 		msg_gerr("Out of memory!\n");				\
-		exit(1);						\
+		return -999;						\
 	}								\
 	undo_pci_write_data->dev = *a;					\
 	undo_pci_write_data->reg = b;					\
-- 
1.7.6

